/* MGH is a trademark of Morris G. Hirsch.
 * Morris G. Hirsch is a trademark of Morris G. Hirsch.
 *
 * Morris G. Hirsch ASSUMES NO RESPONSIBILITY FOR THE USE,
 * OR INABILITY TO USE ANY OF ITS SOFTWARE.
 * MGH SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
 * AND MGH EXPRESSLY DISCLAIMS ALL IMPLIED WARRANTIES,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. */

/* Unix (tm) stuff */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

/* X / Motif (tm) stuff */

#include <X11/X.h>
#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <X11/StringDefs.h>
#include <X11/cursorfont.h>
#include <X11/Xutil.h>
#include <X11/MwmUtil.h>
#include <X11/Xlib.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <Xm/DialogS.h>
#include <Xm/BulletinB.h>
#include <Xm/RowColumn.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/MainW.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/SelectioB.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/ScrolledW.h>
#include <Xm/ScrollBar.h>
#include <Xm/MenuShell.h>

int map_grid;
int map_cities;
char map_overlays[1000];

Widget    hSB, vSB;

Widget    gridlabel, citylabel, overlabel;

/* The scrollbar value ranges from min to max minus slider,
   and thus represents the left or bottom not center of slider.
   Avoid the runtime diagnostic for attempted motion beyond limits!
   The notify True calls any VC callbacks as if user made change. */

void scroll_command(int horiz, int vert)
{
   int increment, pageIncrement,
   minimum, maximum,
   value, sliderSize;

  XtVaGetValues(hSB,
  XmNvalue, &value,
  XmNminimum, &minimum,
  XmNmaximum, &maximum,
  XmNsliderSize, &sliderSize,
  XmNincrement, &increment,
  XmNpageIncrement, &pageIncrement,
  NULL);

  if ((value + horiz > minimum)
   && (value + horiz < maximum - sliderSize))
    XmScrollBarSetValues(hSB,
    value +horiz, sliderSize, increment, pageIncrement,
    True);

  XtVaGetValues(vSB,
  XmNvalue, &value,
  XmNminimum, &minimum,
  XmNmaximum, &maximum,
  XmNsliderSize, &sliderSize,
  XmNincrement, &increment,
  XmNpageIncrement, &pageIncrement,
  NULL);

  if ((value + vert > minimum)
   && (value + vert < maximum - sliderSize))
    XmScrollBarSetValues(vSB,
    value +vert, sliderSize, increment, pageIncrement,
    True);
}

Widget 
demo_make_label(Widget parent,
char *string, Position posx, Position posy)
{
  Widget      widget;
  XmString    tcs;
  Arg       args[10];
  int       nn = 0;

  nn = 0;

  if (posx || posy) {
    XtSetArg(args[nn], XmNx, posx); nn++;
    XtSetArg(args[nn], XmNy, posy); nn++;
  }

/* At least one space now. */

  if (string && strlen(string))
    tcs = XmStringLtoRCreate(string, XmSTRING_DEFAULT_CHARSET);
  else
    tcs = XmStringLtoRCreate(" ", XmSTRING_DEFAULT_CHARSET);

  XtSetArg(args[nn], XmNrecomputeSize, True); nn++;
  XtSetArg(args[nn], XmNlabelString, tcs); nn++;

  widget = XmCreateLabel(parent, "Label", args, nn);

  XmStringFree(tcs);

  return (widget);
}

void 
demo_set_string(Widget w,
char *string)
{
  int  nn;
  Arg       args[10];
  XmString    tcs;

  nn = 0;

  if (XtIsSubclass(w, xmLabelWidgetClass)
   || XtIsSubclass(w, xmLabelGadgetClass)) {
    tcs = XmStringLtoRCreate(string, XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[nn], XmNlabelString, tcs); nn++;
    XtSetValues(w, args, nn);
    XmStringFree(tcs);
  }
  else if (XtClass(w) == xmTextFieldWidgetClass) {
    XmTextFieldSetString(w, string);
  }
  else if (XtClass(w) == xmTextWidgetClass) {
    XmTextSetString(w, string);
  }
  else
    printf("ERROR: %s is not Label or Text\n", XtName(w));
}

/* Make a new command line reflecting all current settings. */

char * map_save_cmd(void)
{ static char cmdline[1000];

  strcpy(cmdline, "map");

  if (map_grid)
    strcat(cmdline, " -grid");
  else
    strcat(cmdline, " -ngrid");

  if (map_cities == 0)
    strcat(cmdline, " -nc");
  else if (map_cities == 1)
    strcat(cmdline, " -cm");
  else
    strcat(cmdline, " -cn");

  strcat(cmdline, " -o ");
  strcat(cmdline, map_overlays);

  return cmdline;
}

/* Set display and flags per a command vector.
   Start beyond arg zero -- we already know the name. */

void map_parse_cmd(int argc, char *argv[])
{ int ii;

  if (!argv)
    return;

  for (ii = 1; ii < argc; ii++) {

/* show latitude / longitude grid? */

    if (!strcmp(argv[ii], "-ngrid")) {
      demo_set_string(gridlabel, "no grid");
      map_grid = 0;
    }

    else if (!strcmp(argv[ii], "-grid")) {
      demo_set_string(gridlabel, "grid");
      map_grid = 1;
    }

/* show cities: no / markers / names */

    else if (!strcmp(argv[ii], "-nc")) {
      demo_set_string(citylabel, "no cities");
      map_cities = 0;
    }

    else if (!strcmp(argv[ii], "-cm")) {
      demo_set_string(citylabel, "city markers");
      map_cities = 1;
    }

    else if (!strcmp(argv[ii], "-cn")) {
      demo_set_string(citylabel, "city names");
      map_cities = 2;
    }

    else if (!strcmp(argv[ii], "-o")) {
      map_overlays[0] = NULL;
      while ((ii+1 < argc) && (argv[ii+1][0] != '-')) {
        strcat(map_overlays, " ");
        if (wordcount(listops(argv[ii+1],
                              '*',
                              "water roads states"))
            > 0)
          strcat(map_overlays, argv[++ii]);
      }
      demo_set_string(overlabel, map_overlays);
    }

    else
      printf("arg # %d %s is not recognized\n", ii, argv[ii]);
  }
}

/* Make the display agree with flags. */

void map_update(void)
{

/* show latitude / longitude grid? */

    if (map_grid)
      demo_set_string(gridlabel, "grid");
    else
      demo_set_string(gridlabel, "no grid");

/* show cities: no / markers / names */

    if (map_cities == 0)
      demo_set_string(citylabel, "no cities");
    else if (map_cities == 1)
      demo_set_string(citylabel, "city markers");
    else
      demo_set_string(citylabel, "city names");

    demo_set_string(overlabel, map_overlays);
}

static void mypro_CB(Widget w,
 XtPointer client_data, XmAnyCallbackStruct *XACS)
{ printf("I got called for Atom number %d\n", (long)client_data); }

main(int argc, char *argv[])

{ static char   kws[] = "@(#)map.c	1.1 ARGOS 93/12/22 14:46:26";

  char   buff[200];

  XtAppContext  app_context;

  Widget    toplevel, mainwindow, frame, swindow, drawing, menubar;

  int       ii;

  Atom      mypro;

  int       nn = 0;
  Arg       args[10];

/* Application class "Demos" determines where to look for help files.
   See the Makefile. */

  toplevel = XtAppInitialize(&app_context, "Demos",
                 NULL, 0,
                 &argc, argv,
                 NULL,
                 args, nn);

/* Make a MainWindow,
   put our simulated display in the work window,
   and let gui runtime fill the MenuBar as specified. */

  XtManageChild(mainwindow =
          XmCreateMainWindow(toplevel, "mainwindow", NULL, 0));

/* If XmNscrollingPolicy is XmAUTOMATIC,
   it creates the scrollers but shows them only as needed,
   and shows the single DA child through a clipping window. */

  XtManageChild (frame =
    XmCreateFrame (mainwindow, "frame", NULL, 0));

/* Create a drawing window inside ScrolledWindow.
   If AUTOMATIC the scrolling behavior will be done for us.
   The DrawingArea XmNresizePolicy resource has several options.
   When RESIZE_ANY there is no point in setting w and h values.
   If the dimensions exceed actual main window scrollers appear. */

  nn = 0;
  XtSetArg (args[nn], XmNscrollingPolicy, XmAUTOMATIC);  nn++;
  XtManageChild (swindow =
    XmCreateScrolledWindow (frame, "swindow", args, nn));

/* Positions would bump the span out,
   if resizePolicy is XmRESIZE_ANY or _GROW. */

  nn = 0;
  XtSetArg(args[nn], XmNwidth, 500); nn++;
  XtSetArg(args[nn], XmNheight, 600); nn++;
  XtSetArg(args[nn], XmNresizePolicy, XmRESIZE_NONE); nn++;

/* The simulated display... */

  XtManageChild(drawing =
    XmCreateDrawingArea(swindow, "drawing", args, nn));

  XtManageChild(gridlabel =
    demo_make_label(drawing, "gridmode", 150, 200));

  XtManageChild(citylabel =
    demo_make_label(drawing, "citymode", 100, 150));

  strcpy(map_overlays, "water roads states");
  XtManageChild(overlabel =
    demo_make_label(drawing, map_overlays, 50, 100));

/* We will "push" these to pan in response to menu commands. */

  XtVaGetValues(swindow,
   XmNverticalScrollBar, &vSB,
   XmNhorizontalScrollBar, &hSB,
   NULL);

  XtManageChild(menubar =
          XmCreateMenuBar(mainwindow, "menubar", NULL, 0));

/* Two things to know about this call:
   If the UI spec has a TOOLBAR it will add it as the XmNcommandWindow.
   The other NULL args here don't hurt existing scrollbars. */

  XmMainWindowSetAreas(mainwindow,
                       menubar,
                       NULL, NULL, NULL,
                       frame);

  XtRealizeWidget(toplevel);

/* Populate and show the GUI. */

  gui_init(menubar);

/* Additions to the WM menu are ALL via ONE formatted string.
   These are "silly" examples because no better ones come to mind.
   All that belongs here are inter-client negotiation for screen space.
   Notice any ClientMessage number needs a callback registered. */

/* -------
  mypro = XmInternAtom(XtDisplay(mainwindow), "myPro", False);
  sprintf(buff, "my\\ Protocol _P Ctrl<Key>V f.send_msg %d", mypro);

  strcat(buff, "\nBeep _B Ctrl<Key>B f.beep");

  strcat(buff, "\nDup f.exec ");
  for (ii = 0; ii < argc; ii++)
  { strcat(buff, argv[ii]); strcat(buff, " "); }

  cx_addmenu(buff);
  cx_menu_CB(mypro, mypro_CB);
--------- */

  gui_show();

/* Handle any args remaining after X has had a chance. */

  map_parse_cmd(argc, argv);

  XtAppMainLoop(app_context);
}
