/* Application call to update a property on toplevel shell windows,
   but only anywhere it is already defined. */

#include <X11/X.h>
#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <X11/StringDefs.h>
#include <X11/Xutil.h>
#include <X11/Xlib.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/Protocols.h>

extern Display      *cx_display;
extern Screen       *cx_screen;
extern Window        cx_rootwindow;

extern Widget        cx_toplevel;
extern Window        cx_topwindow;

int cx_bcast(char *property_name,
 int new_format, char *new_data, long new_nitems)

{ static char   kws[] = "@(#)cx_bcast.c	1.1 ARGOS 93/11/12 17:31:49";

 Atom prop_atom = XmInternAtom(cx_display, property_name, False);

 Window root_window;
 Window parent_window;
 Window *child_windows;
 unsigned int num_children;
 int ii;

 Atom actual_type;
 int actual_format;
 unsigned long nitems;
 unsigned long leftover_bytes;
 unsigned char **data = NULL;

 int sent = 0;

/* Note return Status 0 failure / 1 success. */

   if (!XQueryTree(cx_display, cx_rootwindow,
                   &root_window,
                   &parent_window,
                   &child_windows,
                   &num_children))
       return 0;

   for (ii = 0; ii < num_children; ii++) {

/* Test each one for already having prop_atom.
   Just allow for one word regardless of actual size;
   we only care if it exists yet.
   Note return 1 failure / Success success. */

     if (XGetWindowProperty(cx_display, child_windows[ii],
        prop_atom, 0L, 1L, False, AnyPropertyType,
        &actual_type,
        &actual_format,
        &nitems,
        &leftover_bytes,
        &data) == Success) {

       if(data)
         XFree((char *)data);

/* Use the actual_type / actual_format / new_data / new_nitems. */

       XChangeProperty(cx_display, child_windows[ii],
                   prop_atom,
                   actual_type,
                   actual_format,
                   PropModeReplace,
                   (unsigned char *)&new_data, new_nitems);
       sent++;
     }
   }
   XFree(child_windows);
   return sent;
}
