#include <sys/types.h>

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include <X11/X.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xutil.h>

#include <Xm/Xm.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>


void
cx_set_string(Widget w, char *new_string)

{ static char   kws[] = "@(#)cx_utils.c	1.2 ARGOS 94/03/02 07:45:14";

  int       nn = 0;
  Arg       args[10];
  XmString    tcs;
  char       *current;
  int       ip;

/* c y a */

  if (!w)
    return;

  if (!new_string)
    return;

/* Labels and Buttons */

  if (XtIsSubclass(w, xmLabelWidgetClass)
    || XtIsSubclass(w, xmLabelGadgetClass)) {
    tcs = XmStringLtoRCreate(new_string, XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[nn], XmNlabelString, tcs);
    nn++;
    XtSetValues(w, args, nn);
    XmStringFree(tcs);
  }

/* Replace only if differs to avoid blinking of same value. */

  else if (XtClass(w) == xmTextFieldWidgetClass) {
    current = XmTextFieldGetString(w);
    if (strcmp(current, new_string)) {
      ip = XmTextFieldGetInsertionPosition(w);
      XmTextFieldSetString(w, new_string);
      XmTextFieldSetInsertionPosition(w, ip);
    }
    XtFree(current);
  }

  else if (XtClass(w) == xmTextWidgetClass) {
    current = XmTextGetString(w);
    if (strcmp(current, new_string)) {
      ip = XmTextGetInsertionPosition(w);
      XmTextSetString(w, new_string);
      XmTextSetInsertionPosition(w, ip);
    }
    XtFree(current);
  }
}

/* Ring bell on the server.
   (That is where the user is sitting!)
   Callable from application or internals.
   There is more to setting pitch, duration, and volume,
   all with an "if possible" disclaimer. */

static int    bell_percent = 100;

void
cx_ring_bell(Display *display)
{
  if (!display)
    return;

  XBell(display, bell_percent);
}

/* Swap the current fg and bg resources.
   Caller must keep track of which is normal mode. */

void
cx_reverse(Widget w)
{
  Pixel       bg_col, fg_col;

/* c y a */

  if (!w || !XtIsWidget(w))
    return;

  XtVaGetValues(w,
          XmNforeground, &fg_col,
          XmNbackground, &bg_col,
          NULL);

  XtVaSetValues(w,
          XmNforeground, bg_col,
          XmNbackground, fg_col,
          NULL);
}

/* Get the label string if any otherwise the name. */

char  *
cx_get_string(Widget w)
{
  int       nn = 0;
  Arg       args[10];
  XmString    labelstring = NULL;
  unsigned char   labeltype = NULL;
  static char  *current = NULL;

  if (current)
    XtFree(current);

  current = NULL;

/* c y a */

  if (!w)
    return NULL;

/* Labels and Buttons of all kinds. */

  if (XtIsSubclass(w, xmLabelWidgetClass)
    || XtIsSubclass(w, xmLabelGadgetClass)) {
    XtSetArg(args[nn], XmNlabelType, &labeltype);
    nn++;
    XtSetArg(args[nn], XmNlabelString, &labelstring);
    nn++;
    XtGetValues(w, args, nn);

    if ((labeltype == XmSTRING) && labelstring) {
      if (!XmStringGetLtoR(labelstring,
                 XmSTRING_DEFAULT_CHARSET,
                 &current))
        current = NULL;
    }
  }

  if (labelstring)
    XmStringFree(labelstring);

  if (!current)
    current = strdup(XtName(w));

  return current;
}

/* This is somewhat like XtNametoWidget,
   but takes only the 'final' widget name,
   runs down multiple child branches,
   and allows unknown intermediate steps.
   Specify a class if the name may appear more than once,
   as a wrapper and the widget inside. */

Widget
cx_find_named_child(Widget w,
           char *name, WidgetClass wclass)
{
  int       nn = 0;
  Arg       args[10];
  Cardinal    numchildren;
  WidgetList    children;
  Widget      child;
  int       ii;

/* c y a */

  if (!w || !name || !strlen(name))
    return NULL;

/* Are we there yet?
   Name must match,
   notice we ignore case for this test,
   and either no class specified,
   or matching. */

  if (!strcasecmp(XtName(w), name)
    && (!wclass || XtIsSubclass(w, wclass)))
    return w;

  if (XtIsComposite(w)) {
    nn = 0;
    XtSetArg(args[nn], XtNchildren, &children);
    nn++;
    XtSetArg(args[nn], XtNnumChildren, &numchildren);
    nn++;
    XtGetValues(w, args, nn);

    for (ii = 0; ii < numchildren; ii++)
      if (child = cx_find_named_child(children[ii], name,
                       wclass))
        return child;
  }
  return NULL;
}

/* Will use first upper case letter or digit
   found in the Label or Name as Mnemonic.
   Some places it will work and some it will not.

   Best to set both labels and mnemonics via resources,
   or code both in the program.
   We have program derive it from Label if any,
   else from Name as default. */

static char
cxm_fuc(char *name)
{
  if (!name)
    return NULL;

  while (*name && !isupper(*name) && !isdigit(*name))
    name++;

  return *name;
}

/* if there is a mnemonic key,
   it works when in a menu when visible,
   and is ignored otherwise. */

Widget
cx_set_mnemonic(Widget w)
{
  char       *label_or_name;
  char      mnemonic;

  if (!w)
    return NULL;

  if (XtIsSubclass(w, xmLabelWidgetClass)
    || XtIsSubclass(w, xmLabelGadgetClass)) {
    label_or_name = cx_get_string(w);
    if (mnemonic = cxm_fuc(label_or_name))
      XtVaSetValues(w, XmNmnemonic, mnemonic, NULL);
  }
  return w;
}

Boolean
cx_is_on_off(Widget w)
{
  return ((XtClass(w) == xmToggleButtonGadgetClass)
      || (XtClass(w) == xmToggleButtonWidgetClass));
}

void
cx_set_on_off(Widget w, Boolean on_off, Boolean do_callback)
{
  if (XtClass(w) == xmToggleButtonGadgetClass) {
    XmToggleButtonGadgetSetState(w, on_off, do_callback);
  }

  else if (XtClass(w) == xmToggleButtonWidgetClass) {
    XmToggleButtonSetState(w, on_off, do_callback);
  }
}

/* The toggle callback has state but use this any other time. */

Boolean
cx_get_on_off(Widget w)
{
  if (XtClass(w) == xmToggleButtonGadgetClass) {
    return XmToggleButtonGadgetGetState(w);
  }

  else if (XtClass(w) == xmToggleButtonWidgetClass) {
    return XmToggleButtonGetState(w);
  }
}
