#include "gui_comm.h"

void uiappend(void)

/* Append the current child dictionary to parent dictionary.
   Any duplicate in current replaces that in parent.
   Parent then becomes child valid dictionary.
   (The node and dictionary active level is popped.) */

{ static char   vers[] = "@(#)uiappend.c	1.1 ARGOS 93/12/13 17:37:12";

  UIDATA     *child;
  UIDATA     *nextchild;
  UIDATA     *parent;
  UIDATA     *nextparent;

#ifdef DEBUG
  if (DEBUG) {
    fprintf(DEBUG,
        "uiappend from %s %d to %s %d\n",
        Node_Name(UITOP), UITOP,
        Node_Name(UIPARENT), UIPARENT);
    fflush(DEBUG);
  }
#endif

/* Loop on all contents of the child dictionary.
   Notice we decremented active level at start of the loop,
   so the parent is now the top level. */

/* This could be written to loop and call def for each entry,
   letting it replace if found or insert if not.
   It would be much simpler but less efficient than this,
   because this moves pointers where def would make a new copy. */

  for (child = Data[Actives], Data[Actives--] = 0;
     child;
     child = nextchild) {

/* Save this link before we may lose it. */

    nextchild = Data_Sibling(child);

/* Should this be discarded rather than appended?
   This could be made to test the level instead of simple flag,
   so any data could be propagated only down to specified level. */

    if (Data_NoAppend(child)) {

      switch (Data_Kind(child)) {
      case STRINGVAL:
      case STRUCTVAL:
      case MALLOCVAL:
        free(Data_SValue(child));
        break;
      }

#ifdef DEBUG
      if (DEBUG) {
        fprintf(DEBUG,
            "uiappend discarding %s\n",
            Data_Name(child));
        fflush(DEBUG);
      }
#endif

      free(Data_Name(child));
      free(child);
    }

/* Compare this name to each entry in parent dictionary.
   When they match replace the parent value by child value,
   free the discarded child descriptor and name string,
   and free replaced parent value if it also uses allocated memory. */

    else {
      for (parent = Data[Actives];
         parent;
         parent = nextparent) {

        nextparent = Data_Sibling(parent);

        if (strcasecmp(Data_Name(child), Data_Name(parent)) == 0) {

/* Free memory if same names but different kinds,
   such that the parent value allocated memory but the child did not. */

          if (((Data_Kind(parent) == STRINGVAL)
             || (Data_Kind(parent) == STRUCTVAL)
             || (Data_Kind(parent) == MALLOCVAL))
            && ((Data_Kind(child) != STRINGVAL)
              && (Data_Kind(child) != STRUCTVAL)
              && (Data_Kind(child) != MALLOCVAL))) {
            free(Data_SValue(parent));
            Data_SValue(parent) = NULL;
          }

/* Value and sequence of new (child) replace old (parent) entry. */

          switch (Data_Kind(child)) {
          case LONGVAL:

            if (Data_LValue(parent) != Data_LValue(child)) {
              Data_LValue(parent) = Data_LValue(child);
              Data_SeqStamp(parent) = Data_SeqStamp(child);
#ifdef DEBUG
              if (DEBUG) {
                fprintf(DEBUG,
                    "uiappend replace %s as L=%d\n",
                    Data_Name(child), Data_LValue(child));
                fflush(DEBUG);
              }
#endif
            }
            break;

          case FLOATVAL:
            if (Data_FValue(parent) != Data_FValue(child)) {
              Data_FValue(parent) = Data_FValue(child);
              Data_SeqStamp(parent) = Data_SeqStamp(child);
#ifdef DEBUG
              if (DEBUG) {
                fprintf(DEBUG,
                    "uiappend replace %s as F=%f\n",
                    Data_Name(child), Data_FValue(child));
                fflush(DEBUG);
              }
#endif
            }
            break;

          case DOUBLEVAL:
            if (Data_DValue(parent) != Data_DValue(child)) {
              Data_DValue(parent) = Data_DValue(child);
              Data_SeqStamp(parent) = Data_SeqStamp(child);
#ifdef DEBUG
              if (DEBUG) {
                fprintf(DEBUG,
                    "uiappend replace %s as D=%lf\n",
                    Data_Name(child), Data_DValue(child));
                fflush(DEBUG);
              }
#endif
            }
            break;

/* Both parent and child values use allocated memory.
   Free parent value if different pointers regardless of content. */

          default:
            if (Data_SValue(parent) != Data_SValue(child)) {
              Data_SeqStamp(parent) = Data_SeqStamp(child);
              if (Data_SValue(parent))
                free(Data_SValue(parent));
              Data_SValue(parent) = Data_SValue(child);
            }
#ifdef DEBUG
            if (DEBUG) {
              if (Data_Kind(child) == STRINGVAL)
                fprintf(DEBUG,
                  "uiappend replace %s as S=%s\n",
                  Data_Name(child), Data_SValue(child));
              else
                fprintf(DEBUG,
                  "uiappend replace %s as U=%d\n",
                  Data_Name(child), Data_SValue(child));
              fflush(DEBUG);
            }
#endif
            break;
          }
          free(Data_Name(child));
          free(child);
          break;;
        }
      }

/* When no match found just insert to (front of) parent dictionary. */

      if (!parent) {
        Data_Sibling(child) = Data[Actives];
        Data[Actives] = child;
      }
    }
  }
}
