#include <string.h>

#include "gui_comm.h"

UIDATA * uiinitval(char *name, int termkind)

/* Search for the named node as a terminal on the active stack.
   Use initial value attribute code to define the value.
   Look further if no initial value attribute.

   Notice how we use it!
   Use init val to define the value in current top dictionary,
   regardless of where the attribute code is found.

   Ancestor dictionaries are not changed until node completion,
   not at all if node cancels. */

{ static char     vers[] = "@(#)uiinitval.c	1.1 ARGOS 93/12/13 17:37:15";

    NODEID          child;

    int             level;
    NODEID          ii = 0;

    long            ltemp;

    char           *chtemp;

    int      *DIE = 0;

    if (!name || !strlen(name)) {
        printf("uiinitval:null or empty name\n");
        *DIE = 0;
    }

    else if (Actives < 1) {
        printf("uiinitval:empty Actives stack for %s\n", name);
        *DIE = 0;
    }

#ifdef DEBUG
    if (DEBUG) {
	fprintf(DEBUG, "uiinitval  %s %d\n", name, termkind);
	fflush(DEBUG);
    }
#endif

/* Search the active levels top down */

    for (level = Actives;
	 level;
	 level--) {

/* Loop for all children of this level */

	for (child = Node_Child(Active_Nodes[level]);
	     child;
	     child = Node_Sibling(child)) {

/* Do not consider any omitted node.
   Only interested in terminals of proper name and kind,
   but we must allow some 'similar' terminal kinds to match. */

	    if (!Node_Omitted(child)
		&& (Node_Kind(child) == TermNode)
		&& (strcasecmp(Node_Name(child), name) == 0)) {

/* Use node if it has appropriate init attribute defined. */

		switch (termkind) {

		case BOOLEAN_TERM:
		case INT_TERM:
		case CYCLE_TERM:
		    if (Node_Long_Init(child)
			&& ((Node_Term_Kind(child) == BOOLEAN_TERM)
			    || (Node_Term_Kind(child) == INT_TERM)
			    || (Node_Term_Kind(child) == CYCLE_TERM))) {
			uidef(name, LONGVAL,
			      Node_Long_Init(child)
			      (child));
			return uiget(name);
		    }
		    break;

		case FLOAT_TERM:
		    if (Node_Float_Init(child)
			&& (Node_Term_Kind(child) == termkind)) {
			uidef(name, FLOATVAL,
			      Node_Float_Init(child)
			      (child));
			return uiget(name);
		    }
		    break;

		case DOUBLE_TERM:
		    if (Node_Double_Init(child)
			&& (Node_Term_Kind(child) == termkind)) {
			uidef(name, DOUBLEVAL,
			      Node_Double_Init(child)
			      (child));
			return uiget(name);
		    }
		    break;

		case STRING_TERM:
		    if (Node_Choice_Init(child)
			&& (Node_Term_Kind(child) == STRING_TERM)) {
			uidef(name, STRINGVAL,
			      Node_Choice_Init(child)
			      (child));
			return uiget(name);
		    }
		    break;

		case CHOICE_TERM:
		    if (Node_Choice_Init(child)
			&& (Node_Term_Kind(child) == CHOICE_TERM)) {
			chtemp = Node_Choice_Init(child)(child);
			if (!strcasecmp(chtemp, "ALL"))
			    chtemp = Node_Choice_Set(child)(child);
			else if (!strcasecmp(chtemp, "NONE"))
			    chtemp = "";
			uidef(name, STRINGVAL, chtemp);
			return uiget(name);
		    }
		    break;

		case STRUCT_TERM:
		    if (Node_Choice_Init(child)
			&& (Node_Term_Kind(child) == termkind)) {

			Node_Term_Decode(child) (Node_Choice_Init(child) (child),
						 0, 0, &ltemp);

			uidef(Node_Name(child),
			      STRUCTVAL, Node_StrucSize(child), ltemp);
		    }
		    break;
		}
	    }
	}
    }

/* Failed to find init attribute for this variable.
   Use the default for each variable type.

   For types that we do not know how to default,
   could detect lack of init at uitoc translate time,
   but such a test may pick up errors that are ok,
   when values are set as side effect actions. */

#ifdef DEBUG
    if (DEBUG) {
	fprintf(DEBUG, "uiinitval defaulting %s\n", name);
	fflush(DEBUG);
    }
#endif

    switch (termkind) {

    case BOOLEAN_TERM:
    case INT_TERM:
    case CYCLE_TERM:
	uidef(name, LONGVAL, 0);
	return uiget(name);
	break;

    case FLOAT_TERM:
	uidef(name, FLOATVAL, 0);
	return uiget(name);
	break;

    case DOUBLE_TERM:
	uidef(name, DOUBLEVAL, 0);
	return uiget(name);
	break;

    case STRING_TERM:
    case CHOICE_TERM:
	uidef(name, STRINGVAL, "");
	return uiget(name);
	break;

    default:
	return 0;
	break;
    }
}
